import json
import jsonschema


CONFIG_SCHEMA = {
    '$schema': 'http://json-schema.org/draft-07/schema#',
    'type': 'object',
    'properties': {
        'SQLALCHEMY_DATABASE_URI': {'type': 'string', 'format': 'database-uri'},
        'SURVEY_DATABASE_URI': {'type': 'string', 'format': 'database-uri'}
    },
    'required': ['SQLALCHEMY_DATABASE_URI', 'SURVEY_DATABASE_URI'],
    'additionalProperties': False
}


def load(f):
    """
    loads, validates and returns configuration parameters

    :param f: file-like object that produces the config file
    :return:
    """
    config = json.loads(f.read())
    jsonschema.validate(config, CONFIG_SCHEMA)
    return config
