"""Add NREN country

Revision ID: 049353edaa79
Revises: 73e493fe7415
Create Date: 2023-05-19 11:02:26.096779

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '049353edaa79'
down_revision = '73e493fe7415'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    # remove some NRENs that shouldn't be in the db
    join_tables = ['budgets', 'funding_source', 'charging_structure', 'nren_staff',
                   'parent_organization', 'sub_organization', 'ec_project', 'policy']

    for join_table in join_tables:
        op.execute(
            f"DELETE FROM {join_table} "
            "WHERE nren_id IN ("
            "  SELECT id FROM nren WHERE UPPER(name) IN ('ARENA', 'BIHARNET', 'SARNET', 'SIGMANET', 'UARNET', 'UNREN')"
            ")"
        )

    op.execute(
        "DELETE FROM nren "
        "WHERE UPPER(nren.name) IN ('ARENA', 'BIHARNET', 'SARNET', 'SIGMANET', 'UARNET', 'UNREN')"
    )

    with op.batch_alter_table('nren', schema=None) as batch_op:
        batch_op.add_column(sa.Column('country', sa.String(), nullable=True))

    countries = {
        'ACOnet': 'Austria',
        'AMRES': 'Serbia',
        'ARNES': 'Slovenia',
        'ASNET-AM': 'Armenia',
        'AzScienceNet': 'Azerbaijan',
        'BASNET': 'Belarus',
        'Belnet': 'Belgium',
        'BREN': 'Bulgaria',
        'CARNET': 'Croatia',
        'CESNET': 'Czech Republic',
        'CYNET': 'Cyprus',
        'DeIC': 'Denmark',
        'DFN': 'Germany',
        'EENet': 'Estonia',
        'FCCN': 'Portugal',
        'Funet': 'Finland',
        'GARR': 'Italy',
        'GRENA': 'Georgia',
        'GRNET S.A.': 'Greece',
        'HEAnet': 'Ireland',
        'IUCC': 'Israel',
        'Jisc': 'United Kingdom',
        'KIFU': 'Hungary',
        'KREN': 'Kosovo',
        'LAT': 'Latvia',
        'LITNET': 'Lithuania',
        'MARnet': 'North Macedonia',
        'MREN': 'Montenegro',
        'PIONIER': 'Poland',
        'RASH': 'Albania',
        'RedIRIS': 'Spain',
        'RENAM': 'Moldova',
        'RENATER': 'France',
        'RESTENA': 'Luxembourg',
        'RHnet': 'Iceland',
        'RoEduNet': 'Romania',
        'SANET': 'Slovakia',
        'SIKT': 'Norway',
        'SUNET': 'Sweden',
        'SURF': 'Netherlands',
        'SWITCH': 'Switzerland',
        'ULAKBIM': 'Turkey',
        'UoM': 'Malta',
        'URAN': 'Ukraine'
    }

    for nren, country in countries.items():
        op.execute(f"UPDATE nren SET country = '{country}' WHERE UPPER(name) = UPPER('{nren}')")

    op.alter_column('nren', 'country', nullable=False)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('nren', schema=None) as batch_op:
        batch_op.drop_column('country')

    op.execute(
        "INSERT INTO nren (name) VALUES "
        "('ARENA'),"
        "('BIHARNET'),"
        "('SARNET'),"
        "('SigmaNet'),"
        "('UARNet'),"
        "('UNREN')"
    )

    # ### end Alembic commands ###
